
var sitegin = require('./sitegin/sitegin');

var input = new sitegin.input.Hugo(process.cwd()+'/content/');
var transformerMD = new sitegin.transformer.Markdown;
var transformerTags = new sitegin.transformer.Tags;
var transformerNunjucks = new sitegin.transformer.Nunjucks;
var transformerUrlizeTags = new sitegin.transformer.UrlizeTags;
var output = new sitegin.output.Filesystem;
var sequencer = new sitegin.Sequencer;
var transformerGit = new sitegin.transformer.Git;
var transformerOther = new sitegin.transformer.Other;
var transformerImages = new sitegin.transformer.Images;

var sass = require('node-sass');
var fs = require('fs');

var builddir = "build";
if(process.argv[2] !== "compileonly") builddir = "build-debug";

var oldEmit = sequencer.emit;
sequencer.emit = function() {
  console.log("Event:",arguments[0]);
  oldEmit.apply(sequencer, arguments);
}

sequencer.on("", function(ev) {
  console.log("Event:",ev);
})

var tasks = [];
var onDone = function(what) {
  console.log(what, 'done');
  var removed = false;
  for(var i = 0; i < tasks.length; i++) {
    if(tasks[i] == what) {
      removed = true;
      tasks.splice(i,1);
      break;
    }
  }
  if(!removed && process.argv[2] == "compileonly") throw new Error("Task "+what+" ended but was never registered");
  if(tasks.length == 0 && process.argv[2] == "compileonly") {
    console.log("exitting");
    process.exit();
  }
}


// PIPELINE
sequencer
.load(input)
.registerNext(transformerGit)
.registerNext(transformerOther)
.registerNext(transformerImages)
.registerNext(transformerMD)
.registerNext(transformerUrlizeTags)
.registerNext(transformerTags)
.registerNext(transformerNunjucks)
.registerNext(output)
.finish(function(list) {
  onDone("sequencer");
})
tasks.push('sequencer');

try {fs.mkdirSync(builddir);}catch(e){}
try {fs.mkdirSync(builddir+"/theme");}catch(e){}

// SASS
var rendersass = function() {
  sass.render ({file: "theme/sass/style.scss"},function(err, result) {
    if(err == null) {
      console.log("compiled sass");
      fs.writeFile(builddir+"/theme/style.css",result.css);
    } else console.log("error ", err);
    onDone("sass");
  });
};
tasks.push('sass');

rendersass();

// STATIC FILES
var fsextra = require('node-fs-extra');
tasks.push('copy static');
fsextra.copy(
    'static',
    builddir,
    function(file){ return !(file.match(".git") || file.match("static/articles")); },
    function(){onDone("copy static");}
  );
tasks.push('copy static/articles');
fsextra.copy(
    'static/articles',
    builddir+"/clanek",
    function(){onDone("copy static/articles");}
  );
tasks.push('copy theme static');
fsextra.copy(
    'theme/static',
    builddir,
    function(file){ return !file.match("\\.git") },
    function(){onDone("copy theme static");}
  );

// DEVEL INCOMING
if(process.argv[2] !== "compileonly") {
  var sync = require('browser-sync').create();
  sync.init({
    server: {
      baseDir: builddir
    },
    ui: {
      port: 1338
    },
    port: 1337
  });

  // CHANGE WATCHER
  var chokidar = require('chokidar');

  chokidar.watch('theme/static', {ignoreInitial: true, usePolling: false})
  .on('all', (event, path) => {
    console.log(path);
    console.log("theme static copy watcher");
    fsextra.copy(
      'theme/static',
      builddir,
      function(file){ return !file.match(".git"); },
      function(){}
      );
    sync.reload(path);
  });
  
  chokidar.watch('static/', {ignoreInitial: true, usePolling: false})
  .on('all', (event, path) => {
    if(path.match(/^static\/articles\/.*$/)) return;
    console.log("static copy watcher");
    fsextra.copy(
      'static',
      builddir,
      function(file){ return !(file.match(".git") || file.match("static/articles")); },
      function(){}
    );
    sync.reload(path);
  })
  .unwatch('static/articles');
  
  chokidar.watch('static/articles', {ignoreInitial: true, usePolling: false})
  .on('all', (event, path) => {
    console.log("static articles copy watcher");
    fsextra.copy('static/articles',builddir+"/clanek",function(){});
    sync.reload(path);
  })
  .unwatch('static/articles');

  chokidar.watch(['theme/','content/'], {ignoreInitial: true, usePolling: false})
  .on('all', (event, path) => {
    if(path.match(/^theme\/static\/.*$/)) return;
    if(path.match(/^theme\/sass\/.*$/)) return;
    console.log("rebuild watcher",path);
    sequencer.load(input);
  })
  .unwatch("theme/static")
  .unwatch("theme/sass");

  sequencer.finish(function() {
    sync.reload("*.html");
  });

  chokidar.watch('theme/sass', {ignoreInitial: true, usePolling: false})
  .on('all', function(event, path) {
    console.log("sass: file change detected - rerendering "+event+" "+path);
    rendersass();
    sync.reload(path);
  });
}

