// Add git commit author and date to metadata
var nodegit = require('nodegit');
var git = require('nodegit-kit');
var path = require('path');
var moment = require('moment');

var pathToRepo = path.resolve("content");

// Returns promise for patch of single commit
var gitShow = function(repo, commit_data) {
  var commit, tree, parentTree;
  return nodegit.Revparse.single(repo, commit_data.commit)
    .then((obj) => {
      return nodegit.Commit.lookup(repo, obj.id());
    })
    .then((_commit) => {
      commit = _commit;
      if(commit.parentcount() < 1) return null;
      return parent = commit.getParents(1);
    })
    .then((parents) => {
      if(parents == null) return null;
      return parents[0].getTree();
    })
    .then((_parentTree) => {
      parentTree = _parentTree;
      return commit.getTree();
    })
    .then((_tree) => {
      tree = _tree;
      return nodegit.Diff.treeToTree(repo, parentTree, tree, null);
    })
    .then((diff) => {
      return diff.patches();
    })
    .then((patches) => {
      return [
        patches,
        commit_data
      ];
    });
}

var getFilesHistory = function() {
  var repo;
  return git.open(pathToRepo)
  .then(function(_repo) {
    repo = _repo;
    return git.log(repo)
  })
  .then(function(log){
    var promises = [];
    for(var i = 0; i < log.length; i++) {
      promises.push(gitShow(repo, log[i]));
    }
    return Promise.all(promises);
  })
  .then(function(diffs) {
    var filehistory = {};
    diffs.forEach(function(data) {
      var commit = data[1];
      var diff = data[0];

      diff.forEach(function(entry) {
        var p = path.normalize(entry.newFile().path());
        if(filehistory[p]===undefined) {
          filehistory[p] = [];
        }
        filehistory[p].push(commit);
      });
    });
    return filehistory;
  });
}

module.exports = function() {
  var tr = this;
  tr.fancyname = "transformer git";

  tr.pre = function(cb, articleList) {
    getFilesHistory()
    .then((filesHistory) => {
      var todo = articleList.length;
      articleList.forEach(function(article) {
        var file = path.relative(pathToRepo, article.origFile);
        var commits = filesHistory[file];
        commits.sort(function(a,b){
          return b.date - a.date;
        });
        if(commits !== undefined) {
          var newestCommit = commits[0];
          var oldestCommit = commits[commits.length-1];

          if(article.metadata.date == undefined)
            article.metadata.date = {};

          if(article.metadata.date.creation == undefined)
            article.metadata.date.creation = oldestCommit.date;

          article.metadata.date.modification = newestCommit.date;
          article.metadata.date.modified = (newestCommit !== oldestCommit);

          if(article.metadata.author === undefined)
            article.metadata.author = {}
          if(article.metadata.author.name == undefined)
            article.metadata.author.name = oldestCommit.author.name
          if(article.metadata.author.email == undefined)
            article.metadata.author.email = oldestCommit.author.email

          article.commits = commits;
        }

        todo--;
        if(todo == 0) cb();
      });
    })
    .catch(function(err) {console.log(err.stack);});
  }

  tr.forEachPage = function(article, cb) {
    cb();
  }
}
