var input = new (require('./input-hugo'));
var lwip;
try {lwip = require('lwip');}
catch(e) {
  lwip = null
  console.log("WARNING! LWIP is not installed - image minification won't work.");
  console.log("You can try to run npm install.");
  console.log("If it doesn't help try to make npm install lwip work.");
}
var path = require('path');

var builddir = "build";
if(process.argv[2] !== "compileonly") builddir = "build-debug";

var minify = function() {
  return Promise.resolve(new Promise(function(resolve, reject) {
    var fileList = [];
    input.walk('static/',
      function(file, stat) {
        if(file.match(/\.png$/i) || file.match(/\.jpe?g$/i)) {
          fileList.push(file);
        }
      },
      function() {
        resolve(fileList);
      }
    );
  })).then(function(fileList) {
    var promises = [];
    fileList.forEach((file) => {
      file = file.split(path.sep).join('/');
      var f = file.split(".");
      var ext = f.splice(-1,1);
      f = f.join(".");
      f = f+"_thumbnail."+ext;
      f = f.split('/');
      f.splice(0,1,builddir);
      f = f.join('/').replace('articles','clanek');

      var minifyPromise = function(resolve, reject) {
        if(lwip) {
          lwip.open(file, function(err, image) {
            if(err) throw err;
            var w = image.width(), h = image.height();
            if(w <= 512) {
              file = file.replace("static/","").replace('articles','clanek');
              resolve({
                minified: false,
                file: file,
                thumb: file
              });
            } else {
              image.batch()
              .resize(512, Math.round(512*h/w))
              .writeFile(f, function(err) {
                if(err) throw err;
                resolve({
                  minified: true,
                  file: file.replace("static/","").replace('articles','clanek'),
                  thumb: f.replace(builddir+'/',"")
                });
              });
            }
          })
        } else {
          file = file.replace("static/","").replace('articles','clanek');
          resolve({
            minified: false,
            file: file,
            thumb: file
          });
        }
      }

      promises.push(new Promise(minifyPromise));
    });
    return Promise.all(promises);
  })
}

module.exports = function() {
  var tr = this;
  tr.fancyname = "transformer images";
  tr.pre = function(cb, list) {
    minify()
    .then(function(imageList) {
      list.images = imageList;
      cb();
    })
    .catch(function(err) {console.log(err.stack); cb();});
  }
}
